from swarms.structs.agent import Agent
from swarms.structs.agents_available import showcase_available_agents
from swarms.structs.auto_swarm import AutoSwarm, AutoSwarmRouter
from swarms.structs.base_structure import BaseStructure
from swarms.structs.base_swarm import BaseSwarm
from swarms.structs.base_workflow import BaseWorkflow
from swarms.structs.concurrent_workflow import ConcurrentWorkflow
from swarms.structs.conversation import Conversation
from swarms.structs.graph_workflow import (
    Edge,
    GraphWorkflow,
    Node,
    NodeType,
)
from swarms.structs.groupchat import GroupChat
from swarms.structs.majority_voting import (
    MajorityVoting,
    majority_voting,
    most_frequent,
    parse_code_completion,
)
from swarms.structs.message import Message
from swarms.structs.mixture_of_agents import MixtureOfAgents
from swarms.structs.multi_agent_collab import MultiAgentCollaboration
from swarms.structs.multi_agent_exec import (
    run_agent_with_timeout,
    run_agents_concurrently,
    run_agents_concurrently_async,
    run_agents_concurrently_multiprocess,
    run_agents_sequentially,
    run_agents_with_different_tasks,
    run_agents_with_resource_monitoring,
    run_agents_with_tasks_concurrently,
    run_single_agent,
)
from swarms.structs.queue_swarm import TaskQueueSwarm
from swarms.structs.rearrange import AgentRearrange, rearrange
from swarms.structs.round_robin import RoundRobinSwarm
from swarms.structs.sequential_workflow import SequentialWorkflow
from swarms.structs.spreadsheet_swarm import SpreadSheetSwarm
from swarms.structs.swarm_arange import SwarmRearrange
from swarms.structs.swarm_net import SwarmNetwork
from swarms.structs.swarm_router import (
    SwarmRouter,
    SwarmType,
    swarm_router,
)
from swarms.structs.swarming_architectures import (
    broadcast,
    circular_swarm,
    exponential_swarm,
    fibonacci_swarm,
    geometric_swarm,
    grid_swarm,
    harmonic_swarm,
    linear_swarm,
    log_swarm,
    mesh_swarm,
    one_to_one,
    one_to_three,
    power_swarm,
    prime_swarm,
    pyramid_swarm,
    sigmoid_swarm,
    staircase_swarm,
    star_swarm,
)
from swarms.structs.task import Task
from swarms.structs.utils import (
    detect_markdown,
    distribute_tasks,
    extract_key_from_json,
    extract_tokens_from_text,
    find_agent_by_id,
    find_token_in_text,
    parse_tasks,
)

__all__ = [
    "Agent",
    "AutoSwarm",
    "AutoSwarmRouter",
    "BaseStructure",
    "BaseSwarm",
    "BaseWorkflow",
    "ConcurrentWorkflow",
    "Conversation",
    "GroupChat",
    "MajorityVoting",
    "majority_voting",
    "most_frequent",
    "parse_code_completion",
    "Message",
    "MultiAgentCollaboration",
    "SwarmNetwork",
    "AgentRearrange",
    "rearrange",
    "RoundRobinSwarm",
    "SequentialWorkflow",
    "Task",
    "detect_markdown",
    "distribute_tasks",
    "extract_key_from_json",
    "extract_tokens_from_text",
    "find_agent_by_id",
    "find_token_in_text",
    "parse_tasks",
    "MixtureOfAgents",
    "GraphWorkflow",
    "Node",
    "NodeType",
    "Edge",
    "broadcast",
    "circular_swarm",
    "exponential_swarm",
    "fibonacci_swarm",
    "geometric_swarm",
    "grid_swarm",
    "harmonic_swarm",
    "linear_swarm",
    "log_swarm",
    "mesh_swarm",
    "one_to_one",
    "one_to_three",
    "power_swarm",
    "prime_swarm",
    "pyramid_swarm",
    "sigmoid_swarm",
    "staircase_swarm",
    "star_swarm",
    "TaskQueueSwarm",
    "SpreadSheetSwarm",
    "SwarmRouter",
    "SwarmType",
    "SwarmRearrange",
    "run_agents_concurrently",
    "run_agents_concurrently_async",
    "run_single_agent",
    "run_agents_concurrently_multiprocess",
    "run_agents_sequentially",
    "run_agents_with_different_tasks",
    "run_agent_with_timeout",
    "run_agents_with_resource_monitoring",
    "swarm_router",
    "run_agents_with_tasks_concurrently",
    "showcase_available_agents",
]



import re


def extract_code_from_markdown(markdown_content: str) -> str:
    """
    Extracts code blocks from a Markdown string and returns them as a single string.

    Args:
    - markdown_content (str): The Markdown content as a string.

    Returns:
    - str: A single string containing all the code blocks separated by newlines.
    """
    # Regular expression for fenced code blocks with optional language specifier
    pattern = r"```(?:\w+\n)?(.*?)```"

    # Check if markdown_content is a string
    if not isinstance(markdown_content, str):
        raise TypeError("markdown_content must be a string")

    # Find all matches of the pattern
    matches = re.finditer(pattern, markdown_content, re.DOTALL)

    # Extract the content inside the backticks
    code_blocks = []
    for match in matches:
        code_block = match.group(1).strip()
        # Remove any leading or trailing whitespace from the code block
        code_block = code_block.strip()
        # Remove any empty lines from the code block
        code_block = "\n".join(
            [line for line in code_block.split("\n") if line.strip()]
        )
        code_blocks.append(code_block)

    # Concatenate all code blocks separated by newlines
    if code_blocks:
        return "\n\n".join(code_blocks)
    else:
        return ""


# example = """
# hello im an agent
# ```bash
# pip install swarms
# ```
# """

# print(extract_code_from_markdown(example))  # Output: { "type": "function", "function": { "name": "fetch_financial_news", "parameters": { "query": "Nvidia news", "num_articles": 5 } } }



import os
import uuid
from loguru import logger


def initialize_logger(log_folder: str = "logs"):

    WORKSPACE_DIR = os.getenv("WORKSPACE_DIR")
    if not os.path.exists(WORKSPACE_DIR):
        os.makedirs(WORKSPACE_DIR)

    # Create a folder within the workspace_dir
    log_folder_path = os.path.join(WORKSPACE_DIR, log_folder)
    if not os.path.exists(log_folder_path):
        os.makedirs(log_folder_path)

    # Generate a unique identifier for the log file
    uuid_for_log = str(uuid.uuid4())
    log_file_path = os.path.join(
        log_folder_path, f"{log_folder}_{uuid_for_log}.log"
    )

    logger.add(
        log_file_path,
        level="INFO",
        colorize=True,
        backtrace=True,
        diagnose=True,
        enqueue=True,
        retention="10 days",
        compression="zip",
    )
    return logger
